/*
NOME: 0008_DENSTPO_EXPORTA_NF_FATURAMENTO.SQL

DESCRIO: Procedimento para gerao das NFs de sadas geradas pelo sistema Faturamento
e notas fiscais de devoluo.

CRIAO: 25/08/2008 - Fabrcio Marcelo da Silva

@C:\Web_550\sql\end_conv\oracle\clientes\den\0008_DENSTPO_EXPORTA_NF_FATURAMENTO.SQL

RAISE_APPLICATION_ERROR(-20501, '' , TRUE);
*/

CREATE OR REPLACE PROCEDURE EXPORTA_NF_FATURAMENTO(
     P_DTINICIAL IN NOTAFISCAL.DTENTRSAID%TYPE,
     P_DTFINAL   IN NOTAFISCAL.DTENTRSAID%TYPE)
AS
   V_DTGRV         DATE;
   V_HRGRV         VARCHAR2(04);

   V_FILIAL        VARCHAR2(10);
   V_FORMPGTO      VARCHAR2(3);

   V_NUM6          INTEGER;

   V_NOVOCODIGO    INTEGER;
   V_NRSEQ         VARCHAR2(03);
BEGIN
   V_DTGRV    := NULL;
   V_HRGRV    := NULL;

   V_NRSEQ := '001';

   /* Retorna data e hora de exportao */
   BEGIN
      SELECT TO_DATE(SYSDATE), TO_CHAR(SYSDATE,'HH24MI')
      INTO V_DTGRV, V_HRGRV
      FROM DUAL;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_DTGRV := NULL;
         V_HRGRV := NULL;
   END;

   /* Verificao dos documentos de sada (Faturamento) */
   FOR C_SAIDA IN (SELECT N.CDFILIAL,
                          N.NRLANCTONF,
                          N.CDFILIEMIT,
                          T.IDORIGDESTOP,
                          LPAD(TRUNC(N.NRNOTAFISC),10,'0') NRNOTAFISC,
                          N.DTENTRSAID,
                          SUBSTR(replace(replace(replace(N.NRINSJURNF, '-', ''),'.',''),'/',''),1,14) AS NRINSJURNF,
                          LPAD(FO.CDFORMPGTO,3,'0') AS CDFORMPGTO,
                          N.IDSTATUS, N.CDCLIENTE,
                          USR_DADOS_ADIC(N.CDFILIAL, N.NRLANCTONF) AS DSDADOSADIC,
                          'N' AS TIPO,
                          '' AS NFORIG
                     FROM NOTAFISCAL N, CLIENTE FO, TIPOOPERAC T,
                          (SELECT DISTINCT(CDFILIAL)
                             FROM FILGRUPFILI
                            WHERE USR_IDPROTHEUS = 'S') F
                    WHERE (N.IDENTRSAIDOP = 'S')
                      AND (N.DTENTRSAID BETWEEN P_DTINICIAL AND P_DTFINAL)
                      AND (N.IDSTATUS IN ('A','E'))
                      AND (NVL(N.IDSISTEMA,'DFS') IN ('DFS','FAI'))
                      AND (N.CDTIPOOPER <> '06')
                      AND (N.USR_DTEXPORTACAO IS NULL)
                      AND (N.CDCLIENTE = FO.CDCLIENTE (+))
                      AND (N.IDENTRSAIDOP = T.IDENTRSAIDOP)
                      AND (N.CDTIPOOPER = T.CDTIPOOPER)
                      AND (N.CDFILIEMIT = F.CDFILIAL)
                   UNION ALL
                   SELECT N.CDFILIAL,
                          N.NRLANCTONF,
                          N.CDFILIEMIT,
                          T.IDORIGDESTOP,
                          LPAD(TRUNC(N.NRNOTAFISC),10,'0') NRNOTAFISC,
                          N.DTENTRSAID,
                          SUBSTR(replace(replace(replace(N.NRINSJURNF, '-', ''),'.',''),'/',''),1,14) AS NRINSJURNF,
                          LPAD(FO.CDFORMPGTO,3,'0') AS CDFORMPGTO,
                          N.IDSTATUS, N.CDCLIENTE,
                          USR_DADOS_ADIC(N.CDFILIAL, N.NRLANCTONF) AS DSDADOSADIC,
                          'D' AS TIPO,
                          NT.NRNOTAFISC AS NFORIG
                     FROM NOTAFISCAL N, FORNECEDOR FO, NOTAFISCAL NT, TIPOOPERAC T,
                          (SELECT DISTINCT(CDFILIAL)
                             FROM FILGRUPFILI
                            WHERE USR_IDPROTHEUS = 'S') F,
                          (SELECT N.CDFILIAL, N.NRLANCTONF,
                                  MAX(I.CDFILIALIT) AS CDFILIALIT, MAX(I.NRLANCTONFIT) AS NRLANCTONFIT
                             FROM NOTAFISCAL N, ITEMNOTAFISC I
                             WHERE (N.IDENTRSAIDOP = 'S')
                               AND (N.DTENTRSAID BETWEEN P_DTINICIAL AND P_DTFINAL)
                               AND (N.IDSTATUS IN ('A','E'))
                               AND (NVL(N.IDSISTEMA,'DFS') IN ('DFS','FAI'))
                               AND (N.CDTIPOOPER = '06')
                               AND (N.USR_DTEXPORTACAO IS NULL)
                               AND (I.CDFILIAL = N.CDFILIAL)
                               AND (I.NRLANCTONF = N.NRLANCTONF)
                             GROUP BY N.CDFILIAL, N.NRLANCTONF) IT
                    WHERE (N.IDENTRSAIDOP = 'S')
                      AND (N.DTENTRSAID BETWEEN P_DTINICIAL AND P_DTFINAL)
                      AND (N.IDSTATUS IN ('A','E'))
                      AND (NVL(N.IDSISTEMA,'DFS') IN ('DFS','FAI'))
                      AND (N.CDTIPOOPER = '06')
                      AND (N.USR_DTEXPORTACAO IS NULL)
                      AND (N.CDFORNECED = FO.CDFORNECED)
                      AND (N.CDFILIAL = IT.CDFILIAL)
                      AND (N.NRLANCTONF = IT.NRLANCTONF)
                      AND (NT.CDFILIAL = IT.CDFILIALIT)
                      AND (NT.NRLANCTONF = IT.NRLANCTONFIT)
                      AND (N.IDENTRSAIDOP = T.IDENTRSAIDOP)
                      AND (N.CDTIPOOPER = T.CDTIPOOPER)
                      AND (N.CDFILIEMIT = F.CDFILIAL))
   LOOP
      V_FILIAL   := NULL;
      V_FORMPGTO := NULL;

      SELECT MAX(CDGRUPFILI)
        INTO V_FILIAL
        FROM FILGRUPFILI
       WHERE CDFILIAL = C_SAIDA.CDFILIEMIT
         AND USR_IDPROTHEUS = 'S';

      IF NOT V_FILIAL IS NULL THEN

         /* Verifica forma de pagamento para o cliente */
         IF C_SAIDA.IDORIGDESTOP = 'C' THEN
            SELECT LPAD(MAX(F.CDFORMPGTO),3,'0')
              INTO V_FORMPGTO
              FROM CLIENTE F
             WHERE F.CDCLIENTE = C_SAIDA.CDCLIENTE;
         ELSE
            IF C_SAIDA.IDORIGDESTOP = 'F' THEN
               V_FORMPGTO := C_SAIDA.CDFORMPGTO;
            END IF;
         END IF;

         /* Verifica existncia do pedido de venda */
         SELECT NVL(MAX(W6_IDAFPVC),0)
           INTO V_NUM6
           FROM USR_ZW6
          WHERE CDFILIAL = C_SAIDA.CDFILIAL
            AND NRLANCTONF = C_SAIDA.NRLANCTONF
            AND ORIGPEDIDO = 'N'
            AND W6_TPEDI = '1';

         /* Retorna o sequencial para a tabela */
         BEGIN
            SELECT W6_IDAFPVC.NEXTVAL
              INTO V_NOVOCODIGO
              FROM DUAL;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_NOVOCODIGO := 1;
         END;

         IF V_NUM6 = 0 THEN
            INSERT INTO USR_ZW6
                (CDFILDEST, CDFILLANC, NRPEDEXOL, W6_IDAFPVC,
                 W6_EMPRESA, W6_FILIAL, W6_NUM,
                 W6_CGC, W6_CC, W6_CONDPAG, W6_TIPO, W6_TPEDI,
                 ORIGPEDIDO, CDFILIAL, NRLANCTONF, MENSNF,
                 W6_DTGRV, W6_HRGRV, W6_DTIMP, W6_HRIMP)
            VALUES
                (C_SAIDA.CDFILIEMIT, C_SAIDA.CDFILIEMIT, C_SAIDA.NRNOTAFISC, V_NOVOCODIGO,
                 SUBSTR(V_FILIAL, 7, 2), SUBSTR(V_FILIAL, 9, 2), SUBSTR(C_SAIDA.NRNOTAFISC, 5, 6),
                 C_SAIDA.NRINSJURNF, C_SAIDA.CDFILIEMIT, V_FORMPGTO, C_SAIDA.TIPO, '1',
                 'N', C_SAIDA.CDFILIAL, C_SAIDA.NRLANCTONF, C_SAIDA.DSDADOSADIC,
                 V_DTGRV, V_HRGRV, NULL, '');
         ELSE
            INSERT INTO USR_ZW6
                (CDFILDEST, CDFILLANC, NRPEDEXOL, W6_IDAFPVC,
                 W6_EMPRESA, W6_FILIAL, W6_NUM,
                 W6_CGC, W6_CC, W6_CONDPAG, W6_TIPO, W6_TPEDI,
                 ORIGPEDIDO, CDFILIAL, NRLANCTONF, MENSNF,
                 W6_DTGRV, W6_HRGRV, W6_DTIMP, W6_HRIMP)
            VALUES
                (C_SAIDA.CDFILIEMIT, C_SAIDA.CDFILIEMIT, C_SAIDA.NRNOTAFISC, V_NOVOCODIGO,
                 SUBSTR(V_FILIAL, 7, 2), SUBSTR(V_FILIAL, 9, 2), SUBSTR(C_SAIDA.NRNOTAFISC, 5, 6),
                 C_SAIDA.NRINSJURNF, C_SAIDA.CDFILIEMIT, V_FORMPGTO, C_SAIDA.TIPO, '2',
                 'N', C_SAIDA.CDFILIAL, C_SAIDA.NRLANCTONF, C_SAIDA.DSDADOSADIC,
                 V_DTGRV, V_HRGRV, NULL, '');

            FOR C_ITEXCL IN (SELECT CDPRODUTO, DTPROGENPEOL,
                                    W7_EMPRESA, W7_FILIAL, W7_NUM,
                                    W7_CGC, W7_ITEM, W7_PRODUTO, W7_CC, W7_QTDVEN,
                                    W7_PRCVEN, W7_PRUNIT, W7_VALOR, W7_NFORIG
                               FROM USR_ZW7
                              WHERE CDFILIAL = C_SAIDA.CDFILIAL
                                AND NRLANCTONF = C_SAIDA.NRLANCTONF
                                AND W7_IDAFPVI = V_NUM6
                              ORDER BY CDPRODUTO, DTPROGENPEOL) LOOP

               INSERT INTO USR_ZW7
                    (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO, DTPROGENPEOL,
                     W7_IDAFPVI, W7_EMPRESA, W7_FILIAL, W7_NUM,
                     W7_CGC, W7_ITEM, W7_PRODUTO, W7_CC, W7_QTDVEN,
                     W7_PRCVEN, W7_PRUNIT, W7_VALOR, W7_DTENTR, W7_TPEDI,
                     CDFILIAL, NRLANCTONF,
                     W7_DTGRV, W7_HRGRV, W7_DTIMP, W7_HRIMP, W7_NFORIG)
               VALUES
                    (C_SAIDA.CDFILIEMIT, C_SAIDA.CDFILIEMIT, C_SAIDA.NRNOTAFISC,
                     C_ITEXCL.CDPRODUTO, C_ITEXCL.DTPROGENPEOL,
                     V_NOVOCODIGO, C_ITEXCL.W7_EMPRESA, C_ITEXCL.W7_FILIAL, C_ITEXCL.W7_NUM,
                     C_ITEXCL.W7_CGC, C_ITEXCL.W7_ITEM, C_ITEXCL.W7_PRODUTO, C_ITEXCL.W7_CC, C_ITEXCL.W7_QTDVEN,
                     C_ITEXCL.W7_PRCVEN, C_ITEXCL.W7_PRUNIT, C_ITEXCL.W7_VALOR, C_ITEXCL.DTPROGENPEOL, '2',
                     C_SAIDA.CDFILIAL, C_SAIDA.NRLANCTONF,
                     V_DTGRV, V_HRGRV, NULL, '', C_ITEXCL.W7_NFORIG);
            END LOOP;

            /* Retorna o sequencial para a tabela */
            BEGIN
               SELECT W6_IDAFPVC.NEXTVAL
                 INTO V_NOVOCODIGO
                 FROM DUAL;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_NOVOCODIGO := 1;
            END;

            INSERT INTO USR_ZW6
                (CDFILDEST, CDFILLANC, NRPEDEXOL, W6_IDAFPVC,
                 W6_EMPRESA, W6_FILIAL, W6_NUM,
                 W6_CGC, W6_CC, W6_CONDPAG, W6_TIPO, W6_TPEDI,
                 ORIGPEDIDO, CDFILIAL, NRLANCTONF, MENSNF,
                 W6_DTGRV, W6_HRGRV, W6_DTIMP, W6_HRIMP)
            VALUES
                (C_SAIDA.CDFILIEMIT, C_SAIDA.CDFILIEMIT, C_SAIDA.NRNOTAFISC, V_NOVOCODIGO,
                 SUBSTR(V_FILIAL, 7, 2), SUBSTR(V_FILIAL, 9, 2), SUBSTR(C_SAIDA.NRNOTAFISC, 5, 6),
                 C_SAIDA.NRINSJURNF, C_SAIDA.CDFILIEMIT, V_FORMPGTO, C_SAIDA.TIPO, '1',
                 'N', C_SAIDA.CDFILIAL, C_SAIDA.NRLANCTONF, C_SAIDA.DSDADOSADIC,
                 V_DTGRV, V_HRGRV, NULL, '');
         END IF;

         FOR C_ITEMSAIDA IN (SELECT I.CDPRODUTO, P.CDARVPROD, ROUND(I.QTITEMPROD, 2) AS QTITEMPROD,
                                    ROUND(I.VRUNITITEM, 4) AS VRUNITITEM, ROUND(I.VRTOTAITEM, 2) AS VRTOTAITEM
                               FROM ITEMPRODUTO I, PRODUTO P
                              WHERE I.CDFILIAL = C_SAIDA.CDFILIAL
                                AND I.NRLANCTONF = C_SAIDA.NRLANCTONF
                                AND I.CDPRODUTO = P.CDPRODUTO)
         LOOP
            INSERT INTO USR_ZW7
                 (CDFILDEST, CDFILLANC, NRPEDEXOL, CDPRODUTO,
                  DTPROGENPEOL, W7_IDAFPVI, W7_EMPRESA, W7_FILIAL,
                  W7_NUM, W7_CGC, W7_ITEM, W7_PRODUTO,
                  W7_CC, W7_QTDVEN, W7_PRCVEN, W7_PRUNIT,
                  W7_VALOR, W7_DTENTR, W7_TPEDI, CDFILIAL, NRLANCTONF,
                  W7_DTGRV, W7_HRGRV, W7_DTIMP, W7_HRIMP, W7_NFORIG)
            VALUES
                 (C_SAIDA.CDFILIEMIT, C_SAIDA.CDFILIEMIT, C_SAIDA.NRNOTAFISC, C_ITEMSAIDA.CDPRODUTO,
                  NULL, V_NOVOCODIGO, SUBSTR(V_FILIAL, 7, 2), SUBSTR(V_FILIAL, 9, 2),
                  SUBSTR(C_SAIDA.NRNOTAFISC, 5, 6), C_SAIDA.NRINSJURNF, V_NRSEQ, SUBSTR(C_ITEMSAIDA.CDARVPROD, 1, 15),
                  NULL, C_ITEMSAIDA.QTITEMPROD, C_ITEMSAIDA.VRUNITITEM, C_ITEMSAIDA.VRUNITITEM,
                  C_ITEMSAIDA.VRTOTAITEM, NULL, '1', C_SAIDA.CDFILIAL, C_SAIDA.NRLANCTONF,
                  V_DTGRV, V_HRGRV, NULL, '', C_SAIDA.NFORIG);

            V_NRSEQ := LPAD(RTRIM(TO_CHAR(TO_NUMBER(RTRIM(V_NRSEQ))+ 1)),3,'0');
         END LOOP;

         /* Inclui a data de exportao para os documentos exportados */
         UPDATE NOTAFISCAL
            SET USR_DTEXPORTACAO = V_DTGRV
          WHERE CDFILIAL = C_SAIDA.CDFILIAL
            AND NRLANCTONF = C_SAIDA.NRLANCTONF;

         V_NRSEQ := '001';
      END IF;
   END LOOP;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('EXPORTA_NF_FATURAMENTO','5.50.XXX'); END;
/